<?php

/**
 * Requirements tracker, for javascript and css.
 * @todo Document the requirements tracker, and discuss it with the others.
 * 
 * @package framework
 * @subpackage view
 */
class Compressor {
	
	public static $_backend = null;
	
	public static function get_backend(){
		if(self::$_backend == null){
			self::$_backend = new Compressor_Backend();
		}
		return self::$_backend;
	}
	
	public static function javascripts(array $jss){
		foreach($jss as $js){
			Requirements::javascript($js);
		}
		return true;
	}
	
	public static function compress_file($filePath,$fileList,$includeAfterCompress = true){
		return self::get_backend()->compress_file($filePath,$fileList,$includeAfterCompress);
	}
}

class Compressor_Backend {
	
	public static $_combine_js_with_jsmin = true;
	protected $_blocked = array();
	
	public function compress_file($filePath,$fileList,$includeAfterCompress = true){
		$base = Director::baseFolder() . '/';
		$combinedFilePath = $base.$filePath;
		// Make the folder if necessary
		if(!file_exists(dirname($combinedFilePath))) {
			Filesystem::makeFolder(dirname($combinedFilePath));
		}
	
		// If the file isn't writeable, don't even bother trying to make the combined file and return (falls back to uncombined)
		// Complex test because is_writable fails if the file doesn't exist yet.
		if(
			(file_exists($combinedFilePath) && !is_writable($combinedFilePath))
			|| (!file_exists($combinedFilePath) && !is_writable(dirname($combinedFilePath)))
		){
			user_error("Compressor::compress_file(): Couldn't create '$combinedFilePath'", E_USER_WARNING);
			return false;
		}
	
		// Determine if we need to build the combined include
		if(file_exists($combinedFilePath) && !isset($_GET['flush'])) {
			// file exists, check modification date of every contained file
			$srcLastMod = 0;
			foreach($fileList as $file) {
				if(!file_exists($base.$file)){continue;}
				$srcLastMod = max(filemtime($base.$file), $srcLastMod);
			}
			$refresh = $srcLastMod > filemtime($combinedFilePath);
		} else {
			// file doesn't exist, or refresh was explicitly required
			$refresh = true;
		}
		
		// block already included
		
		foreach(array_diff($fileList,$this->_blocked) as $file) {
			$this->_blocked[$file] = true;
		}
		
		if(!$refresh){
			if($includeAfterCompress){
				$isJS = stripos($file, '.js');
				if($isJS){
					Requirements::javascript($filePath);
				}else{
					Requirements::css($filePath);
				}
			}
			return false;
		}
		
		// collect combined data
		$combinedData = "";
		foreach(array_diff($fileList,$this->_blocked) as $file) {
			if(!file_exists($base.$file)){continue;} // check if file exists
			$fileContent = file_get_contents($base.$file);
			$isJS = stripos($file, '.js');
			if($isJS && $this->_combine_js_with_jsmin) {
				//require_once('thirdparty/jsmin/jsmin.php');
				require_once(Director::baseFolder().'/'.ZZ_BASIC_PATH.'/thirdparty/minify/JSMinPlus.php');
				increase_time_limit_to();
				$fileContent = JSMinPlus::minify($fileContent);
				//$fileContent = JSMin::minify($fileContent);
			} else {
				$isCSS = stripos($file,'.css');
				if($isCSS) {
					require_once(Director::baseFolder().'/'.ZZ_BASIC_PATH.'/thirdparty/minify/CSS/Compressor.php');
					increase_time_limit_to();
					$fileContent = Minify_CSS_Compressor::process($fileContent);
				}				
			}
			// write a header comment for each file for easier identification and debugging
			// also the semicolon between each file is required for jQuery to be combinable properly
			$combinedData .= "/****** FILE: $file *****/\n" . $fileContent . "\n".($isJS ? ';' : '')."\n";
		}
		// write combined data
		$successfulWrite = false;
		$fh = fopen($combinedFilePath, 'wb');
		if($fh){
			if(fwrite($fh, $combinedData) == strlen($combinedData)) $successfulWrite = true;
			fclose($fh);
			unset($fh);
		}
		// Unsuccessful write - just include the regular JS files, rather than the combined one
		if(!$successfulWrite) {
			user_error("Compressor::compress_file(): Couldn't create '$combinedFilePath'", E_USER_WARNING);
			return false;
		}
		if($includeAfterCompress){
			if($isJS){
				Requirements::javascript($filePath);
			}else{
				Requirements::css($filePath);
			}
		}
		return true;
	}
}