<?php
/**
 * A elRTE-powered WYSIWYG HTML editor field with image and link insertion and tracking capabilities. Editor fields
 * are created from <textarea> tags, which are then converted with JavaScript.
 *
 * elRTE is an open-source WYSIWYG HTML-editor written in JavaScript using jQuery UI. It features rich text editing,
 * options for changing its appearance, style and many more. You can use it in any commercial or non-commercial projects.
 * Main goal of the editor - simplify work with text and formating (HTML) on sites, blogs, forums and other online services.
 * http://elrte.org/
 *
 * Requires JQuery UI 
 *
 * @package zz-basic
 * @subpackage fields-elRTE
 * @author Anton Fedyanin aka Tony Air New Castle LLC (http://thewebdevelopmentagency.com/) 
 */
class ElEditorField extends TextareaField {

	protected $rows = 30;
	protected $buttons = array();	
	/**
	 * Includes the JavaScript neccesary for this field to work using the {@link Requirements} system.
	 * JQuery UI already loaded by HTMLEditorField_Toolbar
	 */
	public static function include_js() {
		$js_scripts = array(
			// Common JS and widgets
			ZZ_BASIC_PATH.'/thirdparty/elrte/ellib/js/eli18n.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/ellib/js/eldialogform.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/ellib/js/jquery.elcolorpicker.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/ellib/js/jquery.elborderselect.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/ellib/js/jquery.elpaddinginput.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/ellib/js/jquery.elselect.js',
			// elRTE core
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/elRTE.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/elRTE.options.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/elRTE.utils.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/elRTE.DOM.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/elRTE.selection.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/elRTE.w3cRange.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/elRTE.ui.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/elRTE.history.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/elRTE.filter.js',
			
			// elRTE ui buttons
			//ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/anchor.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/blockquote.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/copy.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/css.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/div.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/docstructure.js',
			//ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/elfinder.js',
			// formating management
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/fontname.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/fontsize.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/forecolor.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/formatblock.js',
			
			//ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/fullscreen.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/ssui/fullscreen.js',  // SS integration
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/horizontalrule.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/ssui/image.js',  // SS integration
			//ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/image.js',
			//ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/flash.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/indent.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/justifyleft.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/ssui/link.js',  // SS integration
			//ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/link.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/nbsp.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/outdent.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/pasteformattext.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/pastetext.js',
			//ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/save.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/stopfloat.js',
			// table management
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/table.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/tablerm.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/tbcellprops.js',				
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/tbcellsmerge.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/tbcellsplit.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/tbcollbefore.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/tbcolrm.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/tbrowbefore.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/tbrowrm.js',
			
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/unlink.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/undo.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/direction.js',
			//ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/smiley.js', // do u need smiles?
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/pagebreak.js',
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/ui/about.js',
			ZZ_BASIC_PATH.'/backend/javascript/ElEditorField.js', // SS Integration
			//ZZ_BASIC_PATH.'/backend/javascript/ElEditorField_LinkForm.js', // SS Integration
			//ZZ_BASIC_PATH.'/backend/javascript/ElEditorField_MediaForm.js', // SS Integration
		);
		
		$configObj = ElEditorConfig::get_active();
		//Requirements::block(FRAMEWORK_DIR.'/javascript/HtmlEditorField.js');
		//Requirements::block('htmlEditorConfig');
		// compiling editor scripts
		// css
		Compressor::compress_file(
			ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/ellib/comb_elRTE.css',
			array(
				// CSS for widgets
				ZZ_BASIC_PATH.'/thirdparty/elrte/ellib/css/elcommon.css',
				ZZ_BASIC_PATH.'/thirdparty/elrte/ellib/css/elcolorpicker.css',
				ZZ_BASIC_PATH.'/thirdparty/elrte/ellib/css/eldialogform.css',
				ZZ_BASIC_PATH.'/thirdparty/elrte/ellib/css/elpaddinginput.css',
				ZZ_BASIC_PATH.'/thirdparty/elrte/ellib/css/elselect.css',
				ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/css/elrte.css',
			)
		);
		// language
		$js_scripts[] = ZZ_BASIC_PATH.'/thirdparty/elrte/elrte/js/i18n/elrte.'.i18n::get_tinymce_lang().'.js';
		// javascript
		if(Director::isDev()){
			Compressor::javascripts($js_scripts);
		}else{
			Compressor::compress_file(
				ZZ_BASIC_PATH.'/thirdparty/elrte/comb_elRTE.js',
				$js_scripts
			);
		}
		//Requirements::javascripts($js_scripts);
		$js = $configObj->generateJS();
		Requirements::customScript($js,'elRteConfig');
	}
	
	/**
	 * @see TextareaField::__construct()
	 */
	public function __construct($name, $title = null, $value = '') {
		if(count(func_get_args()) > 3) {
			Deprecation::notice('3.0', 'Use setRows() and setColumns() instead of constructor arguments',
				Deprecation::SCOPE_GLOBAL);
		}

		parent::__construct($name, $title, $value);
		
		self::include_js();
	}
	
	/**
	 * @return string
	 */
	public function Field($properties = array()) {
		// mark up broken links
		$value  = new SS_HTMLValue($this->value);
		
		if($links = $value->getElementsByTagName('a')) foreach($links as $link) {
			$matches = array();
			
			if(preg_match('/\[sitetree_link(?:\s*|%20|,)?id=([0-9]+)\]/i', $link->getAttribute('href'), $matches)) {
				if(!DataObject::get_by_id('SiteTree', $matches[1])) {
					$class = $link->getAttribute('class');
					$link->setAttribute('class', ($class ? "$class ss-broken" : 'ss-broken'));
				}
			}

			if(preg_match('/\[file_link(?:\s*|%20|,)?id=([0-9]+)\]/i', $link->getAttribute('href'), $matches)) {
				if(!DataObject::get_by_id('File', $matches[1])) {
					$class = $link->getAttribute('class');
					$link->setAttribute('class', ($class ? "$class ss-broken" : 'ss-broken'));
				}
			}
		}
		if($images = $value->getElementsByTagName('img')) foreach($images as $image) {
			$image->setAttribute('src',Director::absoluteBaseURL().$image->getAttribute('src'));
		}
		
		return $this->createTag (
			'textarea',
			$this->getAttributes(),
			htmlentities($value->getContent(), ENT_COMPAT, 'UTF-8')
		);
	}

	public function getAttributes() {
		$width = isset($this->width)?$this->width:'97%';
		$height = isset($this->height)?$this->height:($this->rows*16).'px';
		return array_merge(
			parent::getAttributes(),
			array(
				'class'	=>	'elRTEArea',
				'style'	=>	'width:'.$width.';height:'.$height, // prevents horizontal scrollbars
			)
		);
	}
	public function setHeight($h){
		$this->height = $h;
	}
	public function saveInto(DataObjectInterface $record) {
		if($record->escapeTypeForField($this->name) != 'xml') {
			throw new Exception (
				'HtmlEditorField->saveInto(): This field should save into a HTMLText or HTMLVarchar field.'
			);
		}
		
		$linkedPages = array();
		$linkedFiles = array();
		
		$htmlValue = new SS_HTMLValue($this->value);
		
		if(class_exists('SiteTree')) {
			// Populate link tracking for internal links & links to asset files.
			if($links = $htmlValue->getElementsByTagName('a')) foreach($links as $link) {
				$href = Director::makeRelative($link->getAttribute('href'));

				if($href) {
					if(preg_match('/\[sitetree_link,id=([0-9]+)\]/i', $href, $matches)) {
						$ID = $matches[1];

						// clear out any broken link classes
						if($class = $link->getAttribute('class')) {
							$link->setAttribute('class',
								preg_replace('/(^ss-broken|ss-broken$| ss-broken )/', null, $class));
						}

						$linkedPages[] = $ID;
						if(!DataObject::get_by_id('SiteTree', $ID))  $record->HasBrokenLink = true;

					} else if(substr($href, 0, strlen(ASSETS_DIR) + 1) == ASSETS_DIR.'/') {
						$candidateFile = File::find(Convert::raw2sql(urldecode($href)));
						if($candidateFile) {
							$linkedFiles[] = $candidateFile->ID;
						} else {
							$record->HasBrokenFile = true;
						}
					} else if($href == '' || $href[0] == '/') {
						$record->HasBrokenLink = true;
					}
				}
			}
			if($images = $htmlValue->getElementsByTagName('img')) foreach($images as $image) {
				$image->setAttribute('src',str_replace(Director::absoluteBaseURL(),'',$image->getAttribute('src')));
			}
		}
		
		// Resample images, add default attributes and add to assets tracking.
		if($images = $htmlValue->getElementsByTagName('img')) foreach($images as $img) {
			// strip any ?r=n data from the src attribute
			$img->setAttribute('src', preg_replace('/([^\?]*)\?r=[0-9]+$/i', '$1', $img->getAttribute('src')));
			if(!$image = File::find($path = urldecode(Director::makeRelative($img->getAttribute('src'))))) {
				if(substr($path, 0, strlen(ASSETS_DIR) + 1) == ASSETS_DIR . '/') {
					$record->HasBrokenFile = true;
				}
				
				continue;
			}
			
			// Resample the images if the width & height have changed.
			$width  = $img->getAttribute('width');
			$height = $img->getAttribute('height');
			
			if($image){
				if($width && $height && ($width != $image->getWidth() || $height != $image->getHeight())) {
					//Make sure that the resized image actually returns an image:
					$resized=$image->ResizedImage($width, $height);
					if($resized) $img->setAttribute('src', $resized->getRelativePath());
				}
			}
			
			// Add default empty title & alt attributes.
			if(!$img->getAttribute('alt')) $img->setAttribute('alt', '');
			if(!$img->getAttribute('title')) $img->setAttribute('title', '');
			
			//If the src attribute is not set, then we won't add this to the list:
			if($img->getAttribute('src')){
				// Add to the tracked files.
				$linkedFiles[] = $image->ID;
			}
		}
		
		// Save file & link tracking data.
		if(class_exists('SiteTree')) {
			if($record->ID && $record->many_many('LinkTracking') && $tracker = $record->LinkTracking()) {
			    $tracker->removeByFilter(sprintf('"FieldName" = \'%s\' AND "SiteTreeID" = %d',
			    	$this->name, $record->ID));

				if($linkedPages) foreach($linkedPages as $item) {
					$SQL_fieldName = Convert::raw2sql($this->name);
					DB::query("INSERT INTO \"SiteTree_LinkTracking\" (\"SiteTreeID\",\"ChildID\", \"FieldName\")
						VALUES ($record->ID, $item, '$SQL_fieldName')");
				}
			}
		
			if($record->ID && $record->many_many('ImageTracking') && $tracker = $record->ImageTracking()) {
			    $tracker->where(
			    	sprintf('"FieldName" = \'%s\' AND "SiteTreeID" = %d', $this->name, $record->ID)
			    )->removeAll();

				$fieldName = $this->name;
				if($linkedFiles) foreach($linkedFiles as $item) {
					$tracker->add($item, array('FieldName' => $this->name));
				}
			}
		}
		
		$record->{$this->name} = $htmlValue->getContent();
	}

	/**
	 * @return ElEditorField_Readonly
	 */
	public function performReadonlyTransformation() {
		$field = new HtmlEditorField_Readonly($this->name, $this->title, $this->value);
		$field->setForm($this->form);
		$field->dontEscape = true;
		return $field;
	}
	
	public function performDisabledTransformation() {
		return $this->performReadonlyTransformation();
	}
}