/**
 * Functions for HtmlEditorFields in the back end.
 * Includes the JS for the ImageUpload forms. 
 * 
 * Relies on the jquery.form.js plugin to power the 
 * ajax / iframe submissions
 * @author "Tony Air" <tony@thewebdevelopmentagency.com>
 * Copyright: New Castle LLC http://thewebdevelopmentagency.com/
 */

 var ss = ss || {};
/**
 * Wrapper for HTML WYSIWYG libraries, which abstracts library internals
 * from interface concerns like inserting and editing links.
 * Caution: Incomplete and unstable API.
 */
var ElEditorSelector = 'textarea.elRTEArea';
 ss.editorWrappers = ss.editorWrappers || {};
 ss.editorWrappers.initial
 ss.editorWrappers.ElEditor = (function($) {
	return {
		init: function(config) {
			ss.editorWrappers['default'] = function(){return ss.editorWrappers.ElEditor;};
			if(!ss.editorWrappers.ElEditor.initialized) {
				ss.editorWrappers.ElEditor.initialized = true;
				$('textarea.elRTEArea').elrte(config); // activate all elRTE fieldsf
			}
		},
		remove: function() {
			if(ss.editorWrappers.ElEditor.initialized) {
				ss.editorWrappers.ElEditor.initialized = false;
				$('textarea.elRTEArea').elrte('destroy'); // destroy all elRTE fields
			}
		},
		/**
		 * @return Mixed Implementation specific object
		 */
		getInstance: function() {
			// get current instance (shall be defined by ElEditorSelector)
			return $(ElEditorSelector).elrte()[0].elrte;
			//return tinyMCE.activeEditor;
		},
		/**
		 * Invoked when a content-modifying UI is opened.
		 */
		onopen: function() {
		},
		/**
		 * Invoked when a content-modifying UI is closed.
		 */
		onclose: function() {
		},
		/**
		 * Write the HTML back to the original text area field.
		 */
		save: function() {
			// all elRTE fields
			$('textarea.elRTEArea').elrte('updateSource');
		},
		/**
		 * Create a new instance based on a textarea field.
		 *
		 * @param String
		 * @param Object Implementation specific configuration
		 * @param Function
		 */
		create: function(domID, config, onSuccess) {
			alert('create');
			//var ed = new tinymce.Editor(domID, config);
			//ed.onInit.add(onSuccess);
			//ed.render();
		},
		/**
		 * Redraw the editor contents
		 */
		repaint: function() {
			this.getInstance().ui.update();
			//tinyMCE.execCommand("mceRepaint");
		},
		/**
		 * @return boolean
		 */
		isDirty: function() {
			alert('dirty');
			return this.getInstance().isDirty();
		},
		/**
		 * HTML representation of the edited content.
		 * 
		 * Returns: {String}
		 */
		getContent: function() {
			//alert('getContent'+this.getInstance().selection.getRangeAt());
			return $(ElEditorSelector).html();
			//return this.getInstance().selection.getRangeAt();
		},
		/**
		 * DOM tree of the edited content
		 * 
		 * Returns: DOMElement
		 */
		getDOM: function() {
			alert('getDOM');
			return this.getInstance().dom;
		},
		/**
		 * Returns: DOMElement
		 */
		getContainer: function() {
			alert('getContainer');
			return this.getInstance().getContainer();
		},
		/**
		 * Get the closest node matching the current selection.
		 * 
		 * Returns: DOMElement
		 */
		getSelectedNode: function() {
			var node = this.getInstance().selection.getNode();
			if(this.getInstance().dom.selfOrParentLink(node)){
				return this.getInstance().dom.selfOrParentLink(node);
			}
			return this.getInstance().selection.selected();
		},
		/**
		 * Select the given node within the editor DOM
		 * 
		 * Parameters: {DOMElement}
		 */
		selectNode: function(node) {
			this.getInstance().selection.select(node);
		},
		/**
		 * Insert content at the current caret position
		 * 
		 * @param String HTML
		 */
		insertContent: function(html, opts) {
			this.getInstance().selection.deleteContents().insertHtml(html);
		},
		/**
		 * Replace currently selected content
		 * 
		 * @param {String} html
		 */
		replaceContent: function(html, opts) {
			this.getInstance().selection.deleteContents().insertHtml(html);
		},
		/**
		 * Insert or update a link in the content area (based on current editor selection)
		 * 
		 * Parameters: {Object} attrs
		 */
		insertLink: function(attrs, opts) {
			var node = this.getSelectedNode();
			if($(node).is('a')){ // check if selected node is link
				$(node).attr(attrs);
			}else{ // if it's not wrap it in link
				var a = document.createElement('a');
				$(a).attr(attrs);
				$(node).wrapInner($(a));
			}
		},
		/**
		 * Remove the link from the currently selected node (if any).
		 */
		removeLink: function() {
			var node = this.getSelectedNode();
			if($(node).is('a')){ // check if selected node is link
				var html = $(node).html();
				this.getInstance().selection.deleteContents().insertHtml(html);
				this.getInstance().ui.update();
			}
		},
		/**
		 * Strip any editor-specific notation from link in order to make it presentable in the UI.
		 * 
		 * Parameters: 
		 *  {Object} 
		 *  {DOMElement}
		 */
		cleanLink: function(href, node) {
			// Turn into relative
			if(href.match(new RegExp('^' + ssElRTEAreaConfig['document_base_url'] + '(.*)$'))) {
				href = RegExp.$1;
			}
			return href;
		},
		/**
		 * Creates a bookmark for the currently selected range,
		 * which can be used to reselect this range at a later point.
		 * @return {mixed}
		 */
		createBookmark: function() {
			return this.getInstance().selection.getBookmark();
		},
		/**
		 * Selects a bookmarked range previously saved through createBookmark().
		 * @param  {mixed} bookmark
		 */
		moveToBookmark: function(bookmark) {
			this.getInstance().selection.moveToBookmark(bookmark);
		},
		/**
		 * Removes any selection & de-focuses this editor
		 */
		blur: function() {
			this.getInstance().selection.collapse();
			this.getInstance().ui.update();
		},
		/**
		 * Add new undo point with the current DOM content.
		 */
		addUndo: function() {
			//alert('addUndo');
			//this.getInstance().undoManager.add();
		}
	};
})(jQuery);
// Override this to switch editor wrappers
ss.editorWrappers['defaultElEditor'] = ss.editorWrappers.ElEditor;


(function($) {
	$.entwine('ssEl',function($){
		$('textarea.elRTEArea').entwine({
			onadd: function() {
				var edClass = this.data('editor') || ss.editorWrappers['defaultElEditor'], ed = edClass;
				this.setEditor(ed);				
				if(typeof ssElRTEAreaConfig != 'undefined') this.redraw();
				this._super();
			},
			fromWindow: {
				onload: function(){
					this.redraw();
				}
			},
			redraw: function() {
				// Create editor instance and render it.
				// jQuery methods, and avoids replicate the script lazyloading which is already in place with jQuery.ondemand.
				var config = ssElRTEAreaConfig, self = this, ed = this.getEditor();
				ed.init(config);
				this._super();
			},
			getEditor: function(){
				var val = this._super();
				if(!val){this.setEditor(val = this.createEditor());}
				return val;
			},
			createEditor: function(){
				return ss.editorWrappers['defaultElEditor'];
			},
			onremove: function() {
				ed = this.getEditor();
				ed.remove();
			},
			openLinkDialog: function() {
				this.openDialog('link');
			},
			openMediaDialog: function() {
				this.openDialog('media');
			},
			openDialog: function(type) {
				// change elRTE editor selector to get right instance
				ElEditorSelector = 'textarea#'+$(this).attr('id');
				var capitalize = function(text) {
					return text.charAt(0).toUpperCase() + text.slice(1).toLowerCase();
				};

				var url = $('#cms-editor-dialogs').data('url' + capitalize(type) + 'form'),
					dialog = $('.htmleditorfield-' + type + 'dialog');

				/*if(dialog.length) {
					dialog.open();
				} else {*/
					// Show a placeholder for instant feedback. Will be replaced with actual
					// form dialog once its loaded.
					dialog = $('<div class="htmleditorfield-dialog htmleditorfield-' + type + 'dialog loading">');
					$('body').append(dialog);
					$.ajax({
						url: url,
						success: function(html) {
							dialog.html(html);
							dialog.trigger('dialogopen');
						}
					});
				//}
			},
			getContainingForm: function(){
				return this.closest('form');
			},

			fromContainingForm: {
				onbeforesubmitform: function(){
					this.getEditor().save();
				}
			}	
		});
		
		/**
		 * Base form implementation for interactions with an editor instance,
		 * mostly geared towards modification and insertion of content.
		 */
		$('form.htmleditorfield-form').entwine({
			Selection: null,
			Bookmark: null,

			setSelection: function(node) {
				return this._super($(node));
			},

			// Wrapper for various HTML editors
			Editor: null,

			onadd: function() {
				// Move title from headline to (jQuery compatible) title attribute
				var titleEl = this.find(':header:first');
				this.getDialog().attr('title', titleEl.text());

				this._super();
			},
			onremove: function() {
				this.setSelection(null);
				this.setBookmark(null);
				this.setEditor(null);

				this._super();
			},
			getDialog: function() {
				// TODO Refactor to listen to form events to remove two-way coupling
				return this.closest('.htmleditorfield-dialog');
			},
			fromDialog: {
				ondialogopen: function(){
					var ed = this.getEditor();
					ed.onopen();

					this.setSelection(ed.getSelectedNode());
					this.setBookmark(ed.createBookmark());

					ed.blur();

					this.find(':input:not(:submit)[data-skip-autofocus!="true"]').filter(':visible:enabled').eq(0).focus();

					this.updateFromEditor();
					this.redraw();
				},
				ondialogclose: function(){
					var ed = this.getEditor();
					ed.onclose();

					ed.moveToBookmark(this.getBookmark());

					this.setSelection(null);
					this.setBookmark(null);

					this.resetFields();
				}
			},
			createEditor: function(){
				return ss.editorWrappers['defaultElEditor'];
			},
			/**
			 * Get the tinyMCE editor
			 */
			getEditor: function(){
				var val = this._super();
				if(!val) {
					this.setEditor(val = this.createEditor());
				}
				return val;
			},
			modifySelection: function(callback) {
				var ed = this.getEditor();

				ed.moveToBookmark(this.getBookmark());
				callback.call(this, ed);

				this.setSelection(ed.getSelectedNode());
				this.setBookmark(ed.createBookmark());

				ed.blur();
			},
			updateFromEditor: function() {
				/* NOP */
			},
			redraw: function() {
				/* NOP */
			},
			resetFields: function() {
				/* NOP */
			}
		});
	});// ent-wine
})(jQuery);